module Raw(readXmlDoc,readXmlFrom,substXmlEntityRefs) where

import Text.XML.HXT.Arrow
import Text.XML.HXT.Arrow.ParserInterface(parseXmlDoc,substXmlEntityRefs)

{- read and parse xml, verbatim, high preservation:
   xml decl or lack of, pi or lack of, dtd or lack of,
   char refs (in normal text), non-xml entity refs (eg nbsp), cdata

   only decode: xml entity refs (lt, gt, amp, quot, apos),
                remove comments, collapse text,
                element attribute values
 -}
readXmlDoc :: Attributes -> String -> IOStateArrow s b XmlTree
readXmlDoc options uri =
    getDocumentContents options uri
    >>>
    ( ( replaceChildren ( ( getAttrValue a_source
			    &&&
			    xshow getChildren
			  )
			  >>>
			  parseXmlDoc
			  >>>
			  filterErrorMsg
			)
	>>>
	setDocumentStatusFromSystemState "parse XML document"
      )
      `when` documentStatusOk
    )
    >>>
    repairPIs
    >>>
    decode

{- like readXmlDoc but takes String arrowly -}
readXmlFrom :: Attributes -> IOStateArrow s String XmlTree
readXmlFrom options = applyA (arr (readXmlDoc options))


{- PI's (and XML decl) are done somewhat wrong:
   extra level of attribute node.
   We repair them. Doc to doc.
-}
repairPIs :: ArrowList a => a XmlTree XmlTree
repairPIs = fromLA (processChildren (setAttrl (getAttrl >>> getChildren) `when` isPi))

{-
decodes: XML entity refs (lt, gt, amp, apos, quot)
         remove comments
         collapse text
         element attribute values: char refs, collapse text
preserves: cdata
           main text: char refs, uncollapsed text
-}
decode :: ArrowList a => a XmlTree XmlTree
decode = fromLA
  (substXmlEntityRefs
   >>>
   processBottomUp ((none `when` isCmt)
                    >>>
                    collapseXText
                    >>>
                    (processAttrl (processChildren transfCharRef
                                   >>>
                                   collapseXText
                                  )
                     `when` isElem
                    )
                   )
  )
